#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# StartOS Device Manager(ydm).
# Copyright (C) 2011 ivali, Inc.
# hechao <hechao@ivali.com>, 2011.

__author__="hechao"
__date__ ="$2011-12-20 16:36:20$"

import gc
import os
from xml.parsers import expat
from hwclass import *

class Device:

    def __init__(self, dev_xml):

        self.description = ''
        self.product = ''
        self.vendor = ''
        self.version = ''
        self.businfo = ''
        self.logicalname = ''
        self.date = ''
        self.serial = ''
        self.capacity = ''
        self.width = ''
        self.clock = ''
        self.slot = ''
        self.size = ''

        self.lev1 = []
        self.lev2 = []
        self.lev3 = []
        self.lev4 = []
        self.cpus = []
        self.cpu = 0
        self.cachenum = 0
        self.cpunum = 0

        self.config = {}
        self.capability = []
        self.attr = {}

        self.dev_type = {}
        self.pcid = {}

        self._parser = expat.ParserCreate()
        self._parser.buffer_size = 102400
        self._parser.StartElementHandler = self.start_handler
        self._parser.CharacterDataHandler = self.data_handler
        self._parser.EndElementHandler = self.end_handler
#        self._parser.returns_unicode = False

        fd = open(dev_xml, 'rb')
        self._parser.ParseFile(fd)
        fd.close()
        
        self.nvme_newdisk=0
        self.check_nvme()

        self.smbios3_motherb = 0
        self.smbios3_biosinfo = 0
        self.smbios3_getinfo()

    #for nvme disk
    def get_disk_size(self, dev):
        parted="parted -l "
        grep=" | grep  "
        awk=" | awk -F ':' '{print $2}' "
        cmd = parted + dev + grep + dev + awk
        try:
            r=os.popen(cmd, 'r', 1)
            res=r.read()
            for line in res.splitlines():
                return line.strip()
        except:
            return ""

    def get_nvme_info(self, dev, line):
        if line.split(":")[0].strip() == "SN":
            #print "序列号   :" + line.split(":")[1].strip()
            self.serial=line.split(":")[1].strip()
        if line.split(":")[0].strip() == "MN":
            #print "厂家     :" + line.split(":")[1].strip()
            #print "设备名称 :" + dev
            self.nvme_newdisk=1
            #self.size=self.get_disk_size(dev)
            #if self.size: 
                #print "容量大小 :" + self.size
            self.vendor=line.split(":")[1].strip()
            self.logicalname=dev
        if line.split(":")[0].strip() == "FW":
            #print "固件     :" + line.split(":")[1].strip()
            self.version=line.split(":")[1].strip()
        if line.split(":")[0].strip() == "SIZE":
            self.size=line.split(":")[1].strip()

    #find out info by nvme id-ctrl
    def check_nvme_dev(self, line):
        dev=line.split("=:")[1]
        if dev.find("/dev/nvme") == 0:
            if os.path.exists(dev):
                cmd = "nvmecheck info" + " "  + dev 
                r=os.popen(cmd, 'r', 1)
                res=r.read()
                for line in res.splitlines():
                    self.get_nvme_info(dev, line)

    def check_nvme(self):
        #get nvme list 0 1 2 ...
        cmd="/usr/sbin/nvmecheck"
        if os.path.exists(cmd):
            result=os.popen('%s list' % cmd)
            res=result.read()
            for line in res.splitlines():
                #clear old
                self.clear()
                self.check_nvme_dev(line)
                #if found new one, add to "disk"
                if self.nvme_newdisk==1:
                    disk = Disk("NVME Disk", self.product, self.vendor, self.version, \
                    self.businfo, self.logicalname, self.serial, self.size, self.config, self.capability)
                    self.dev_type.setdefault((6, "disk"), []).append(disk)
    #end nvme disk 

    #for smbios3 get info
    def do_getinfo(self, line):
        #get info
        f="Manufacturer:"
        if f in line:
            m=line.split(f)
            self.smbios3_motherb = 1
            self.vendor = m[1].strip()
            return m[1].strip()

        f="Product Name:"
        if f in line:
            m=line.split(f)
            self.product = m[1].strip()
            return m[1].strip()

        f="Serial Number:"
        if f in line:
            m=line.split(f)
            #print(m[1].strip())
            self.serial = m[1].strip()
            return m[1].strip()
        return

    def do_cmd_baseboard(self):
        self.clear()
        cmd="/usr/sbin/dmi-smbios3"
        result=os.popen('%s -t 2' % cmd)
        res=result.read()
        for line in res.splitlines():
            self.do_getinfo(line)
        if self.smbios3_motherb == 1:
            motherboard = Motherboard(self.description, self.product, self.vendor, self.version, self.serial)
            self.dev_type.setdefault((2, "motherboard"), []).insert(0, motherboard)

    def do_getbios_info(self, line):
        #get info
        f="Vendor:"
        if f in line:
            m=line.split(f)
            self.smbios3_biosinfo = 1
            self.vendor = m[1].strip()
            return m[1].strip()

        f="Version:"
        if f in line:
            m=line.split(f)
            self.version = m[1].strip()
            return m[1].strip()

        f="Release Date:"
        if f in line:
            m=line.split(f)
            #print(m[1].strip())
            self.date = m[1].strip()
            return m[1].strip()
        return

    def do_cmd_getbios(self):
        self.clear()
        cmd="/usr/sbin/dmi-smbios3"
        result=os.popen('%s -t 0' % cmd)
        res=result.read()
        for line in res.splitlines():
            self.do_getbios_info(line)
        if self.smbios3_biosinfo == 1:
            bios = Bios(self.description, self.product, self.vendor, self.version, \
            self.date, self.size, self.capability)
            self.dev_type.setdefault((2, "motherboard"), []).append(bios)

    def smbios3_getinfo(self):
        smbios3_file="/tmp/ydm-smbios3-flag"
        if os.path.isfile(smbios3_file): 
            f = open(smbios3_file, "r") 
            line = f.readline().strip()
            #compare
            if line == "non":
                print ( "is not smbios3" )
                return
            if line  != "smbios3":
                print ("flag error")
                return
        else:
            return

        #get smbios3 info
        try:
            cmd="/usr/sbin/dmi-smbios3"
            if os.path.exists(cmd):
                self.do_cmd_baseboard()
                self.do_cmd_getbios()
        except:
            print("get smbios3 maybe something wrong!")

    #end smbios3 get info

    def start_handler(self, tag, attrs):

        self.flag = tag

        if tag == "node":
            self.attr = attrs

        elif tag == "setting":
            self.config.setdefault(attrs["id"], attrs["value"])

        elif tag == "capability":
            self.capability.append(attrs["id"])

    def data_handler(self, data):

        if(data == '\n'):
            return
        if(data.isspace()):
            return

        if self.flag == "description":
            self.description = data.strip()

        elif self.flag == "product":
            self.product = data.strip()

        elif self.flag == "vendor":
            self.vendor = data.strip()

        elif self.flag == "businfo":
            self.businfo = data.strip()

        elif self.flag == "logicalname":
            self.logicalname = data.strip()

        elif self.flag == "version":
            self.version = data.strip()

        elif self.flag == "date":
            self.date = data.strip()

        elif self.flag == "serial":
            self.serial = data.strip()

        elif self.flag == "capacity":
            self.capacity = data.strip()

        elif self.flag == "width":
            self.width = data.strip()

        elif self.flag == "clock":
            self.clock = data.strip()

        elif self.flag == "slot":
            self.slot = data.strip()

        elif self.flag == "size":
            self.size = data.strip()

    def end_handler(self, tag):

        if tag == "node":

            if self.attr["class"] == "system":
                system = System(self.description, self.product, self.vendor, self.version, \
                self.serial, self.width, self.config, self.capability)
                self.dev_type.setdefault((0, "system"), []).append(system)
            
            elif self.attr["id"].split(":")[0] == "cpu" and self.attr["class"] == "processor":
                cpu = Cpu(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.serial, self.slot, self.size, self.capacity, self.width, self.clock, self.config, self.capability)
                if self.businfo:
                    self.cpu = 1
                    self.cpunum += 1
                    if (self.cpunum - self.cachenum) > 1:
                        self.cpus.append(cpu)
                    else:
                        self.dev_type.setdefault((1, "cpu"), []).append(cpu)
                    if self.lev1:
                        cache = self.lev1.pop()  
                        self.dev_type.setdefault((1, "cpu"), []).append(cache)
                        self.cpu = 0
                    if self.lev2:
                        cache = self.lev2.pop()  
                        self.dev_type.setdefault((1, "cpu"), []).append(cache)
                    if self.lev3:
                        cache = self.lev3.pop()  
                        self.dev_type.setdefault((1, "cpu"), []).append(cache)
                    if self.lev4:
                        cache = self.lev4.pop()  
                        self.dev_type.setdefault((1, "cpu"), []).append(cache)

            elif self.attr["id"].split(":")[0] == "cache" and self.attr["class"] == "memory":
                if 'data' in self.capability:
                    list1 = self.description.split(" ")
                    strdes = list1[0] + " data "+list1[1]
                    cache = Cache(strdes, self.product, self.vendor, self.version, self.slot, self.size)
                elif 'instruction' in self.capability:
                    list1 = self.description.split(" ")
                    strdes = list1[0] + " instruction "+list1[1]
                    cache = Cache(strdes, self.product, self.vendor, self.version, self.slot, self.size)
                else:
                    cache = Cache(self.description, self.product, self.vendor, self.version, self.slot, self.size)
                level = self.description.split(" ")
                if self.cpu == 0:
                    if level[0] == "L1":
                        self.cachenum += 1
                        self.lev1.append(cache)
                    elif level[0] == "L2":
                        self.lev2.append(cache)
                    elif level[0] == "L3":
                        self.lev3.append(cache)
                    elif level[0] == "L4":
                        self.lev4.append(cache)
                else:
                    if level[0] == "L1":
                        self.cachenum += 1
                        #just for two or more cpus
                        if self.cpus:
                            done = self.cpunum - len(self.cpus)
                            if self.cachenum - done > 0: 
                                cpu = self.cpus.pop()
                                self.dev_type.setdefault((1, "cpu"), []).append(cpu)
                            
                        if (self.cachenum - self.cpunum) > 0:
                            self.lev1.append(cache)
                            self.cpu = 0
                        else:
                            self.dev_type.setdefault((1, "cpu"), []).append(cache)
                    elif level[0] == "L2":
                        self.dev_type.setdefault((1, "cpu"), []).append(cache)
                    elif level[0] == "L3":
                        self.dev_type.setdefault((1, "cpu"), []).append(cache)
                    elif level[0] == "L4":
                        self.dev_type.setdefault((1, "cpu"), []).append(cache)

            elif (self.attr["id"] == "core" or self.attr["id"] == "board") and self.attr["class"] == "bus":
                if ( self.product == "" and self.vendor == "" and self.serial == "" ) != True:
                    motherboard = Motherboard(self.description, self.product, self.vendor, self.version, self.serial)
                    self.dev_type.setdefault((2, "motherboard"), []).append(motherboard)
            
            elif self.attr["id"] == "firmware" and self.attr["class"] == "memory":
                bios = Bios(self.description, self.product, self.vendor, self.version, \
                self.date, self.size, self.capability)
                self.dev_type.setdefault((2, "motherboard"), []).append(bios)

            elif self.attr["id"].split(":")[0] == "memory" and self.attr["class"] == "memory":
                memory = Memory(self.description, self.product, self.vendor, self.version, \
                self.slot, self.size)
                self.dev_type.setdefault((3, "memory"), []).append(memory)

            elif self.attr["id"].split(":")[0] == "bank" and self.attr["class"] == "memory":
                bank = Bank(self.description, self.product, self.vendor, self.version, \
                self.serial, self.slot, self.size, self.width, self.clock)
                self.dev_type.setdefault((3, "memory"), []).append(bank)

            elif self.attr["id"].split(":")[0] == "display" and self.attr["class"] == "display":
                display = Display(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.config, self.capability)
                self.dev_type.setdefault((4, "display"), []).append(display)
                self.pcid[display.pcid] = "display"
                if get_monitor():
                    monitor = Monitor("", "", "", "")
                    self.dev_type.setdefault((5, "monitor"), [monitor])#.append(monitor)

            elif self.attr["id"].split(":")[0] == "disk" and self.attr["class"] == "disk":
                disk = Disk(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.logicalname, self.serial, self.size, self.config, self.capability)
                self.dev_type.setdefault((6, "disk"), []).append(disk)

            elif self.attr["id"].split(":")[0] == "cdrom" and self.attr["class"] == "disk":
                cdrom = Cdrom(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.logicalname, self.config, self.capability)
                self.dev_type.setdefault((7, "cdrom"), []).append(cdrom)

            elif self.attr["class"] == "storage":
                storage = Storage(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.logicalname, self.serial, self.config, self.capability)
                self.dev_type.setdefault((8, "storage"), []).append(storage)
            
            elif (self.attr["class"] == "network") or (self.attr["id"].split(":")[0] == "bridge" \
                and self.attr["class"] == "bridge"):
                network = Network(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.logicalname, self.serial, self.capacity, self.config, self.capability)
                self.dev_type.setdefault((9, "network"), []).append(network)
                self.pcid[network.pcid] = "network"

            elif self.attr["class"] == "multimedia":
                media = Multimedia(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.config, self.capability)
                self.dev_type.setdefault((10, "multimedia"), []).append(media)
                self.pcid[media.pcid] = "multimedia"

            elif self.attr["class"] == "input":
                imput = Imput(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.config, self.capability)
                self.dev_type.setdefault((11, "input"), []).append(imput)
                self.pcid[imput.pcid] = "input"

            elif self.attr["id"].split(":")[0] != "generic" and self.attr["class"] == "generic":
                generic = Generic(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.serial, self.config, self.capability)
                self.dev_type.setdefault((12, "generic"), []).append(generic)
                self.pcid[generic.pcid] = "generic"

            elif self.attr["id"].split(":")[0] != "communication" and self.attr["class"] == "communication":
                modem = Modem(self.description, self.product, self.vendor, self.version, \
                self.businfo, self.serial, self.config, self.capability)
                self.dev_type.setdefault((12, "generic"), []).append(modem)

            elif self.attr["id"].split(":")[0] == "battery" and self.attr["class"] == "power":
                power = Power(self.description, self.product, self.vendor, self.version, \
                self.slot, self.capacity, self.config)
                self.dev_type.setdefault((12, "generic"), []).append(power)
            
            self.clear()

    def clear(self):

        self.description = ''
        self.product = ''
        self.vendor = ''
        self.version = ''
        self.businfo = ''
        self.logicalname = ''
        self.date = ''
        self.serial = ''
        self.capacity = ''
        self.width = ''
        self.clock = ''
        self.slot = ''
        self.size = ''

        self.config = {}
        self.capability = []
        self.attr = {}
        self.nvme_newdisk=0

    def close(self):

        del self._parser
        gc.collect()
